@@ -9,13 +9,14 @@
 ## no critic (RequireUseStrict, RequireUseWarnings)
 package Riak::Light;
 {
-    $Riak::Light::VERSION = '0.099';
+    $Riak::Light::VERSION = '0.09901';
 }
 ## use critic
 
 use 5.010;
 use Riak::Light::PBC;
 use Riak::Light::Driver;
+use MIME::Base64 qw(encode_base64);
 use Type::Params qw(compile);
 use Types::Standard -types;
 use English qw(-no_match_vars );
@@ -40,6 +41,11 @@ has timeout     => ( is => 'ro', isa => Num,  default => sub {0.5} );
 has tcp_nodelay => ( is => 'ro', isa => Bool, default => sub {1} );
 has in_timeout  => ( is => 'lazy', trigger => 1 );
 has out_timeout => ( is => 'lazy', trigger => 1 );
+has client_id   => ( is => 'lazy', isa     => Str );
+
+sub _build_client_id {
+    "perl_riak_light" . encode_base64( int( rand(10737411824) ), '' );
+}
 
 sub _trigger_autodie {
     my ( $self, $value ) = @_;
@@ -113,28 +119,33 @@ sub BUILD {
     $_[0]->driver;
 }
 
-const my $PING        => 'ping';
-const my $GET         => 'get';
-const my $PUT         => 'put';
-const my $DEL         => 'del';
-const my $GET_KEYS    => 'get_keys';
-const my $QUERY_INDEX => 'query_index';
-const my $MAP_REDUCE  => 'map_reduce';
+const my $PING          => 'ping';
+const my $GET           => 'get';
+const my $PUT           => 'put';
+const my $DEL           => 'del';
+const my $GET_KEYS      => 'get_keys';
+const my $QUERY_INDEX   => 'query_index';
+const my $MAP_REDUCE    => 'map_reduce';
+const my $SET_CLIENT_ID => 'set_client_id';
+const my $GET_CLIENT_ID => 'get_client_id';
 
-const my $ERROR_RESPONSE_CODE       => 0;
-const my $GET_RESPONSE_CODE         => 10;
-const my $GET_KEYS_RESPONSE_CODE    => 18;
-const my $MAP_REDUCE_RESPONSE_CODE  => 24;
-const my $QUERY_INDEX_RESPONSE_CODE => 26;
+const my $ERROR_RESPONSE_CODE         => 0;
+const my $GET_RESPONSE_CODE           => 10;
+const my $GET_KEYS_RESPONSE_CODE      => 18;
+const my $MAP_REDUCE_RESPONSE_CODE    => 24;
+const my $QUERY_INDEX_RESPONSE_CODE   => 26;
+const my $GET_CLIENT_ID_RESPONSE_CODE => 4;
 
 const my $CODES => {
-    $PING        => { request_code => 1,  response_code => 2 },
-    $GET         => { request_code => 9,  response_code => 10 },
-    $PUT         => { request_code => 11, response_code => 12 },
-    $DEL         => { request_code => 13, response_code => 14 },
-    $GET_KEYS    => { request_code => 17, response_code => 18 },
-    $MAP_REDUCE  => { request_code => 23, response_code => 24 },
-    $QUERY_INDEX => { request_code => 25, response_code => 26 },
+    $PING          => { request_code => 1,  response_code => 2 },
+    $GET           => { request_code => 9,  response_code => 10 },
+    $PUT           => { request_code => 11, response_code => 12 },
+    $DEL           => { request_code => 13, response_code => 14 },
+    $GET_KEYS      => { request_code => 17, response_code => 18 },
+    $MAP_REDUCE    => { request_code => 23, response_code => 24 },
+    $QUERY_INDEX   => { request_code => 25, response_code => 26 },
+    $GET_CLIENT_ID => { request_code => 3,  response_code => 4 },
+    $SET_CLIENT_ID => { request_code => 5,  response_code => 6 },
 };
 
 const my $DEFAULT_MAX_RESULTS => 100;
@@ -438,6 +449,27 @@ sub map_reduce_raw {
     );
 }
 
+sub get_client_id {
+    my $self = shift;
+
+    $self->_parse_response(
+        operation => $GET_CLIENT_ID,
+        body      => q(),
+    );
+}
+
+sub set_client_id {
+    state $check = compile( Any, Str );
+    my ( $self, $client_id ) = $check->(@_);
+
+    my $body = RpbSetClientIdReq->encode( { client_id => $client_id } );
+
+    $self->_parse_response(
+        operation => $SET_CLIENT_ID,
+        body      => $body,
+    );
+}
+
 sub _parse_response {
     my ( $self, %args ) = @_;
 
@@ -500,6 +532,9 @@ sub _parse_response {
             $operation, $bucket, $key
           );
 
+        $response_code == $GET_CLIENT_ID_RESPONSE_CODE
+          and return $self->_process_get_client_id_response($response_body);
+
         # we have a 'get' response
         $response_code == $GET_RESPONSE_CODE
           and
@@ -563,6 +598,17 @@ sub _parse_response {
 
 }
 
+sub _process_get_client_id_response {
+    my ( $self, $encoded_message ) = @_;
+
+    $self->_process_generic_error( "Undefined Message", 'get client id', '-',
+        '-' )
+      unless ( defined $encoded_message );
+
+    my $decoded_message = RpbGetClientIdResp->decode($encoded_message);
+    $decoded_message->client_id;
+}
+
 sub _process_get_response {
     my ( $self, $encoded_message, $bucket, $key, $should_decode ) = @_;
 
@@ -641,7 +687,7 @@ Riak::Light - Fast and lightweight Perl client for Riak
 
 =head1 VERSION
 
-version 0.099
+version 0.09901
 
 =head1 SYNOPSIS
 
@@ -801,12 +847,24 @@ requests to Riak over PBC interface.
 
 Perform a ping operation. Will return false in case of error (will store in $@).
 
-=head3 is_alive
+=head3 ping
 
   try { $client->ping() } catch { "oops... something is wrong: $_" };
 
 Perform a ping operation. Will die in case of error.
 
+=head3 set_client_id
+
+  $client->set_client_id('foobar');
+
+Set the client id.
+
+=head3 get_client_id
+
+  my $client_id = $client->get_client_id();
+
+Get the client id.
+
 =head3 get
 
   my $value_or_reference = $client->get(bucket => 'key');
